import pytest

from beam import validators


def test_every():
    with pytest.raises(validators.ValidationError):
        validators.IsValidEvery()("every")

    with pytest.raises(validators.ValidationError):
        validators.IsValidEvery()("every 1")

    with pytest.raises(validators.ValidationError):
        validators.IsValidEvery()("every 1 s")

    with pytest.raises(validators.ValidationError):
        validators.IsValidEvery()("every 1 unknown-s")

    for time_values in [1, 10, 100]:
        for unit in validators.IsValidEvery.time_units:
            # should not fail
            validators.IsValidEvery()(f"every {time_values}{unit}")



from typing import Union

from beam.base import AbstractDataLoader
from beam.serializer import AutoscalingMaxRequestLatencyConfiguration
from beam.types import AutoscalingType


class AutoscalingManager(AbstractDataLoader):
    def __init__(self) -> None:
        self.autoscaling_config: Union[
            AutoscalingMaxRequestLatencyConfiguration, None
        ] = None

    def MaxRequestLatency(self, desired_latency: float, max_replicas: int, **_):
        """
        Arguments:
            desired_latency: maximum time (in seconds), you'd like to wait for a request to be processed
            max_replicas: maximum number of parallel workers spun up to handle incoming requests

        Note: MaxRequestLatency autoscaling only makes sense for asynchronous triggers (Webhook/Schedule)
        """
        self.autoscaling_config = AutoscalingMaxRequestLatencyConfiguration(
            desired_latency=desired_latency,
            max_replicas=max_replicas,
            autoscaling_type=AutoscalingType.MaxRequestLatency,
        )

    def dumps(self):
        if self.autoscaling_config is not None:
            return self.autoscaling_config.validate_and_dump()
        else:
            return {}

    def from_config(self, autoscaling_config: Union[dict, None]):
        if autoscaling_config is None:
            return

        if (
            autoscaling_config.get("autoscaling_type")
            == AutoscalingType.MaxRequestLatency
        ):
            self.MaxRequestLatency(**autoscaling_config)



from typing import List

from beam.base import AbstractDataLoader
from beam.serializer import VolumeConfiguration
from beam.types import MountType


class MountManager(AbstractDataLoader):
    def __init__(self) -> None:
        self.persistent_volumes: List[VolumeConfiguration] = []
        self.shared_volumes: List[VolumeConfiguration] = []

    def PersistentVolume(self, name: str, app_path: str, **_):
        self.persistent_volumes.append(
            VolumeConfiguration(
                name=name,
                local_path=None,
                app_path=app_path,
                mount_type=MountType.Persistent,
            )
        )

    def SharedVolume(self, name: str, app_path: str, **_):
        self.shared_volumes.append(
            VolumeConfiguration(
                name=name,
                local_path=None,
                app_path=app_path,
                mount_type=MountType.Shared,
            )
        )

    def dumps(self):
        return [
            *[pv.validate_and_dump() for pv in self.persistent_volumes],
            *[sv.validate_and_dump() for sv in self.shared_volumes],
        ]

    def from_config(self, mounts: List[dict]):
        if mounts is None:
            return

        for m in mounts:
            if m.get("mount_type") == MountType.Persistent:
                self.PersistentVolume(**m)

            elif m.get("mount_type") == MountType.Shared:
                self.SharedVolume(**m)



import beam
from beam.utils.mock import MockAPI


def test_valid_inputs():
    app = beam.App(
        name="some_app",
        cpu=1,
        memory="128mi",
    )

    schema = {
        "input1": beam.Types.String(),
        "input2": beam.Types.Float(),
        "input4": beam.Types.Boolean(),
        "input5": beam.Types.Json(),
    }

    app.Trigger.RestAPI(
        inputs=schema,
        outputs=schema,
        handler="method.py:run",
    )

    mock = MockAPI(app)
    input_output = {
        "input1": "hello world",
        "input2": 1.0,
        "input4": True,
        "input5": {"hello": "world"},
    }

    output, status = mock.call(**input_output)

    assert output == input_output
    assert status == 200


def test_invalid_inputs():
    app = beam.App(
        name="some_app",
        cpu=1,
        memory="128mi",
    )

    schema = {
        "input1": beam.Types.Float(),
        "input2": beam.Types.Boolean(),
        "input3": beam.Types.Json(),
    }

    app.Trigger.RestAPI(
        inputs=schema,
        outputs=schema,
        handler="method.py:run",
    )

    mock = MockAPI(app)
    input_output = {
        "input1": "Not a float",
        "input2": "Not a boolean",
        "input3": "Not a json",
    }

    output, status = mock.call(**input_output)

    assert output == {
        "errors": [
            "input1:invalid_float_value",
            "input2:invalid_boolean_value",
            "input3:invalid_json_str",
        ]
    }
    assert status == 400

    input_output["input1"] = 1.0
    input_output["input2"] = True
    input_output["input3"] = {"hello": "world"}

    output, status = mock.call(**input_output)

    assert output == input_output
    assert status == 200


def test_valid_loader():
    app = beam.App(
        name="some_app",
        cpu=1,
        memory="128mi",
    )

    schema = {
        "input1": beam.Types.Float(),
        "input2": beam.Types.Boolean(),
        "input3": beam.Types.Json(),
    }

    app.Trigger.RestAPI(
        inputs=schema,
        outputs=schema,
        handler="method.py:run",
        loader="method.py:load",
    )

    input_output = {
        "input1": 1.0,
        "input2": True,
        "input3": {"hello": "world"},
    }

    mock = MockAPI(app)
    output, status = mock.call(**input_output)

    assert mock.loader_output == {"loader_output": "hello world"}
    assert output == input_output
    assert status == 200
